import time
import random
import execjs
import socketio
from .action_ai import ActionAI

class MindSpawner:
    def __init__(self, user_id, action_ai_id):
        self.user_id = user_id
        self.action_ai_id = action_ai_id
        self.sio = socketio.Client()
        self.action_ai = ActionAI()
        self.is_ready = False  # ロード完了フラグ

        # サーバーからのデータ受信を設定
        self.sio.on("libraryData", self._initialize_action_ai)
        self.sio.on("updateAgent", self._update_agent_function)  # ここで受信イベントを設定
        self.sio.connect("https://mindspawner-90819a099a6f.herokuapp.com/")  # サーバーのURLに接続
        self.sio.emit("libraryData", {"id": {"user": user_id, "actionai": action_ai_id}})

    def _initialize_action_ai(self, data):
        """サーバーからデータを受信し、エージェントを登録"""

        agents_data = data.get("agents", [])
        for agent_data in agents_data:
            self.action_ai.add_agent(agent_data["id"], agent_data["function"])

        self.is_ready = True  # ロード完了

    def agent(self, agent_id, input_data):
        """指定したエージェントを実行。ロードが完了していなければ None を返す"""
        if not self.is_ready:
            return None  # ロードが完了していない場合は未定義の値を返す

        for agent in self.action_ai.agents:
            if agent.id == agent_id:
                return agent.execute(input_data)
        
        return None  # エージェントが見つからない場合も None を返す
    
    def get_last_updated(self, agent_id):
        """指定したエージェントの最終更新日時を取得"""
        for agent in self.action_ai.agents:
            if agent.id == agent_id:
                return agent.last_update
        return None
    
    def update_agent(self, agent_id, evaluation_score="auto"):
        """指定したエージェントのアップデートをリクエスト"""
        for agent in self.action_ai.agents:
            if agent.id == agent_id:
                payload = {
                    "id": {
                        "user": self.user_id,
                        "actionai": self.action_ai_id,
                        "agent": agent_id,
                    },
                    "score": evaluation_score
                }

                # 🔹 If evaluation_score is "auto", send full history
                if evaluation_score == "auto":
                    payload["history"] = agent.history
                else:
                    # 🔹 If evaluation_score is not "auto", send one random history entry
                    if agent.history:
                        payload["history"] = [random.choice(agent.history)]  # Select one random entry
                    else:
                        payload["history"] = []  # Empty list if no history available

                self.sio.emit("updateAgent", payload)
                return
        
        print(f"エージェント {agent_id} が見つかりませんでした。")

    def _update_agent_function(self, data):
        """サーバーから新しいエージェントの関数を受信し、適用する"""
        agent_id = data.get("agentid")
        new_function = data.get("function")

        for agent in self.action_ai.agents:
            if agent.id == agent_id:
                agent.function = new_function  # 関数を更新
                
                try:
                    # ✅ execjs.compile() が完了するまで `last_updated` は変更しない
                    ctx = execjs.compile(f"""
                        {new_function}

                        function main(input) {{
                            return stateActionHandler(input);
                        }}
                    """)

                    agent.ctx = ctx  # コンパイル成功後に適用
                    agent.last_updated = time.time()  # ✅ コンパイルが完了した後に更新
                    print(f"✅ エージェント {agent_id} が更新されました。")

                except Exception as e:
                    print(f"⚠️ エージェント {agent_id} の関数更新に失敗: {str(e)}")

                return

        print(f"⚠️ エージェント {agent_id} の更新に失敗しました。（該当エージェントなし）")